@@ -31,11 +31,14 @@ struct TickitWindow {
     TickitCursorShape shape;
     bool visible;
   } cursor;
-  bool is_visible;
-  bool is_focused;
-  bool is_closed;
-  bool steal_input;
-  bool focus_child_notify;
+  struct {
+    unsigned int is_root            : 1;
+    unsigned int is_visible         : 1;
+    unsigned int is_focused         : 1;
+    unsigned int is_closed          : 1;
+    unsigned int steal_input        : 1;
+    unsigned int focus_child_notify : 1;
+  };
 
   int refcount;
   struct TickitHooklist hooks;
@@ -216,6 +219,7 @@ static void init_window(TickitWindow *win, TickitWindow *parent, TickitRect rect
   win->cursor.col = 0;
   win->cursor.shape = TICKIT_CURSORSHAPE_BLOCK;
   win->cursor.visible = true;
+  win->is_root = false;
   win->is_visible = true;
   win->is_focused = false;
   win->is_closed = false;
@@ -236,6 +240,7 @@ TickitWindow* tickit_window_new_root(TickitTerm *term)
     return NULL;
 
   init_window(ROOT_AS_WINDOW(root), NULL, (TickitRect) { .top = 0, .left = 0, .lines = lines, .cols = cols });
+  ROOT_AS_WINDOW(root)->is_root = true;
 
   root->term = tickit_term_ref(term);
   root->hierarchy_changes = NULL;
@@ -261,11 +266,15 @@ TickitWindow* tickit_window_new_root(TickitTerm *term)
 
 static TickitRootWindow *_get_root(const TickitWindow *win)
 {
-  const TickitWindow *root = win;
-  while(root->parent) {
-    root = root->parent;
+  while(!win->is_root) {
+    if(!win->parent) {
+      fprintf(stderr, "tickit_window:_get_root: orphaned window win=%p\n", win);
+      abort();
+    }
+
+    win = win->parent;
   }
-  return WINDOW_AS_ROOT(root);
+  return WINDOW_AS_ROOT(win);
 }
 
 TickitWindow *tickit_window_new(TickitWindow *parent, TickitRect rect, TickitWindowFlags flags)
@@ -350,16 +359,18 @@ void tickit_window_destroy(TickitWindow *win)
     TickitWindow *next = child->next;
 
     tickit_window_unref(child);
+    child->parent = NULL;
     child = next;
   }
 
-  _purge_hierarchy_changes(win);
+  if(win->parent)
+    _purge_hierarchy_changes(win);
 
   if(!win->is_closed)
     tickit_window_close(win);
 
   /* Root cleanup */
-  if(!win->parent) {
+  if(win->is_root) {
     TickitRootWindow *root = WINDOW_AS_ROOT(win);
     if(root->damage) {
       tickit_rectset_destroy(root->damage);
@@ -384,6 +395,10 @@ TickitWindow *tickit_window_ref(TickitWindow *win)
 
 void tickit_window_unref(TickitWindow *win)
 {
+  if(win->refcount < 1) {
+    fprintf(stderr, "tickit_window_unref: invalid refcount %d on win=%p\n", win->refcount, win);
+    abort();
+  }
   win->refcount--;
   if(!win->refcount)
     tickit_window_destroy(win);
@@ -536,7 +551,7 @@ void tickit_window_expose(TickitWindow *win, const TickitRect *exposed)
   if(!win->is_visible)
     return;
 
-  if(win->parent) {
+  if(!win->is_root) {
     tickit_rect_translate(&damaged, win->rect.top, win->rect.left);
     tickit_window_expose(win->parent, &damaged);
     return;
